import 'dart:math';

import 'package:flutter/material.dart';
import 'dart:js_interop';


// Points to the same global object as window
/*@JS('globalThis')
external JSObject get globalThis;
*/

extension type DSBarcodeError(JSObject _) implements JSObject {
  external String name;
  external String message;
}

extension type DSBarcodeResultCoord(JSObject _) implements JSObject {
  external int x;
  external int y;
}

extension type DSBarcodeResult(JSObject _) implements JSObject {
  external int bt;
  external String type;
  external JSInt8Array data;
  external String strdata;
  external JSArray<DSBarcodeResultCoord> points;
  external bool barcodeAtPoint;
  external bool previouslyDecoded;
}

/*
// define the callback
typedef OnBarcodeTCallback = void Function(DSBarcodeResult dr);
// declare onBarcode callback
OnBarcodeTCallback onBarcode = (DSBarcodeResult dr) { 
  debugPrint( 'bt: ${dr.bt}, type: ${dr.type}, data: ${dr.data.toString()}, strdata: ${dr.strdata}, barcodeAtPoint: ${dr.barcodeAtPoint}, previouslyDecoded: ${dr.previouslyDecoded}, points: (${dr.points[0].x}, ${dr.points[0].y}) (${dr.points[1].x}, ${dr.points[1].y}) (${dr.points[2].x}, ${dr.points[2].y}) (${dr.points[3].x}, ${dr.points[3].y})' );
};
*/

@JS()
external void embedScanner(int x, int y, int width, int height);
//external void embedScanner(JSFunction drCallback, int x, int y, int width, int height);

@JS()
external void updateScannerPos(int x, int y, int width, int height);

@JS()
external void addScannerCallback(JSString funcName, JSFunction dsFuncCallback);


void main() {
  runApp( MyApp() );
}


class MyApp extends StatelessWidget {
  const MyApp({super.key});

  // This widget is the root of your application.
  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      title: 'Flutter Demo',
      theme: ThemeData(
        colorScheme: ColorScheme.fromSeed(seedColor: Colors.deepPurple),
      ),
      home: const MyHomePage(title: 'Flutter Demo Home Page'),
    );
  }
}

class MyHomePage extends StatefulWidget {
  const MyHomePage({super.key, required this.title});

  final String title;

  @override
  State<MyHomePage> createState() => _MyHomePageState();
}

class _MyHomePageState extends State<MyHomePage> with WidgetsBindingObserver {
  final GlobalKey _key = GlobalKey();

  bool _firstStart = true;
  String _strBarcode = '';

  //DataSymbol.com Barcode Scanner callback function. Fired when a barcode event occurs.
  void onDSBarcode (DSBarcodeResult dr) { 
    //debugPrint( 'bt: ${dr.bt}, type: ${dr.type}, data: ${dr.data.toString()}, strdata: ${dr.strdata}, barcodeAtPoint: ${dr.barcodeAtPoint}, previouslyDecoded: ${dr.previouslyDecoded}, points: (${dr.points[0].x}, ${dr.points[0].y}) (${dr.points[1].x}, ${dr.points[1].y}) (${dr.points[2].x}, ${dr.points[2].y}) (${dr.points[3].x}, ${dr.points[3].y})' );
    _strBarcode = dr.strdata;
    setState( (){} );
  }

  //DataSymbol.com Barcode Scanner callback function. Fired when a scanner error event occurs.
  void onDSError (DSBarcodeError err) { 
    _strBarcode = '${err.name}, ${err.message}';
    setState( (){} );
  }

  Rectangle<int> getBarcodeViewBox() {
    final RenderBox renderBox = _key.currentContext!.findRenderObject() as RenderBox;
    final position = renderBox.localToGlobal(Offset.zero);
    //debugPrint("addPostFrameCallback position : ${position.dx},${position.dy}");
    return Rectangle( position.dx.toInt(), position.dy.toInt(), renderBox.size.width.toInt(), renderBox.size.height.toInt() );
  }

  @override
  void initState() {
    super.initState();

    WidgetsBinding.instance.addObserver(this);
  }

  @override
  void dispose() {
      WidgetsBinding.instance.removeObserver(this);
      super.dispose();
  }

  @override
  void didChangeMetrics() {
    WidgetsBinding.instance.addPostFrameCallback( (timestamp){
      Rectangle rc = getBarcodeViewBox();
      updateScannerPos( rc.left.toInt(), rc.top.toInt(), rc.width.toInt(), rc.height.toInt() );
    });
  }

  void startUpFunction() async {
    if( _firstStart )
    {
      //Embed and Start DataSymbol.com Barcode Scanner

      addScannerCallback('onError'.toJS, onDSError.toJS);
      addScannerCallback('onBarcode'.toJS, onDSBarcode.toJS);

      Rectangle rc = getBarcodeViewBox();
      embedScanner(rc.left.toInt(), rc.top.toInt(), rc.width.toInt(), rc.height.toInt());
    }

    _firstStart = false;
  }

  @override
  Widget build(BuildContext context) {

     WidgetsBinding.instance.addPostFrameCallback((_) => startUpFunction());

     return Scaffold(
      appBar: AppBar(
        backgroundColor: Theme.of(context).colorScheme.inversePrimary,
        title: Text(widget.title),
      ),
      body: Center(
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          children: <Widget>[
            const Text('Barcode:'),
            Text(
              _strBarcode,
              style: Theme.of(context).textTheme.headlineMedium,
            ),

            SizedBox(// Container(
              key: _key,
              width: 640,
              height: 480,
              //color: Colors.grey[300],
              child: Text(
                'Barcode Scanner Loading ...',
              ),
            ),

          ],
        ),
      ),
    );
  }
}
